﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using TXTextControl.Proofing;

namespace txspell_grid
{
    public partial class Form1 : Form
    {
        // private fields and member variables
        private Image imgInfoIcon;
        private bool bSpelling = true;
        private bool bCloseMenu = true;
        private bool bUserDictionaries = false;

        public Form1()
        {
            InitializeComponent();

            // load a sample xml file as our data source
            DataSet ds = new DataSet();
            ds.ReadXml(Assembly.GetExecutingAssembly().GetManifestResourceStream("txspell_grid.data.xml"));

            dataGridView1.DataSource = ds;
            dataGridView1.DataMember = "product";

            txSpellChecker1.Create();

            // load an icon that is used as an indicator when a cell
            // contains misspelled words
            System.IO.Stream file = Assembly.GetExecutingAssembly().GetManifestResourceStream("txspell_grid.info_icon.png");
            imgInfoIcon = Image.FromStream(file);

            foreach (Dictionary dic in txSpellChecker1.Dictionaries)
            {
                if (dic is UserDictionary == false)
                    continue;

                if (((UserDictionary)dic).IsEditable)
                {
                    bUserDictionaries = true;
                    break;
                }
            }
        }

        // the CellPainting event is used to check the content of the
        // cell in order to display a little icon that shows whether the
        // cell contains misspelled words
        private void dataGridView1_CellPainting(object sender, DataGridViewCellPaintingEventArgs e)
        {
            // termination conditions
            if (bSpelling == false ||
                e.RowIndex == -1 ||
                e.ColumnIndex == -1 ||
                dataGridView1.Rows[e.RowIndex].IsNewRow == true ||
                e.Value == null)
                return;

            // check the text of the current cell
            txSpellChecker1.Check(e.Value.ToString());

            if (txSpellChecker1.IncorrectWords.Count == 0)
            {
                dataGridView1[e.ColumnIndex, e.RowIndex].Tag = null;
                return;
            }
            else
            {
                // tag the cell when misspelled
                // Tag contains the IncorrectWords collection 
                dataGridView1[e.ColumnIndex, e.RowIndex].Tag = txSpellChecker1.IncorrectWords;

                using (
                    Brush gridBrush = new SolidBrush(this.dataGridView1.GridColor),
                    backColorBrush = new SolidBrush(e.CellStyle.BackColor))
                {
                    using (Pen gridLinePen = new Pen(gridBrush))
                    {
                        e.Graphics.FillRectangle(backColorBrush, e.CellBounds);
                        e.PaintBackground(e.ClipBounds, true);
                        e.PaintContent(e.ClipBounds);
                        
                        // draw the little yellow icon in the upper left corner
                        e.Graphics.DrawImage(imgInfoIcon, new Point(e.CellBounds.Left, e.CellBounds.Top));

                        e.Handled = true;
                    }
                }
            }
        }

        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            bSpelling = toolStripButton1.Checked;
            dataGridView1.Refresh();
        }

        private void dataGridView1_CellMouseDown(object sender, DataGridViewCellMouseEventArgs e)
        {
            // delete context menu items
            for (int i = 2; i < contextMenuStrip1.Items.Count - 2; )
            {
                contextMenuStrip1.Items.RemoveAt(i);
            }

            // termination conditions
            if (e.ColumnIndex == -1 ||
                e.RowIndex == -1 ||
                dataGridView1[e.ColumnIndex, e.RowIndex].Tag == null || 
                e.Button != MouseButtons.Right)
            {
                bCloseMenu = false;
                contextMenuStrip1.Close();
                return;
            }

            // create a dynamic context menu that contains all
            // misspelled words and the suggestions in a submenu
            foreach (IncorrectWord word in 
                (IncorrectWordCollection)dataGridView1[e.ColumnIndex, e.RowIndex].Tag)
            {
                ToolStripMenuItem item = new ToolStripMenuItem(word.Text);
                item.Tag = word;
                item.Font = new Font(item.Font.FontFamily,
                    item.Font.Size, 
                    FontStyle.Bold);

                contextMenuStrip1.Items.Insert(2, item);
                
                // create 5 suggestions
                txSpellChecker1.CreateSuggestions(word.Text, 5);

                foreach (Suggestion sug in txSpellChecker1.Suggestions)
                {
                    ToolStripItem subItem = item.DropDownItems.Add(sug.Text);
                    subItem.Tag = dataGridView1[e.ColumnIndex, e.RowIndex];
                    subItem.Click += new EventHandler(item_Click);
                }

                if (txSpellChecker1.Suggestions.Count == 0)
                {
                    ToolStripItem subItem = item.DropDownItems.Add("No suggestions found.");
                }

                item.DropDownItems.Add(new ToolStripSeparator());

                // 'Add To Dictionary' item
                ToolStripMenuItem mnuItemDict = new ToolStripMenuItem("&Add To Dictionary");
                mnuItemDict.Tag = dataGridView1[e.ColumnIndex, e.RowIndex];
                mnuItemDict.Font = new Font(item.Font.FontFamily, 
                    item.Font.Size, 
                    FontStyle.Regular);

                item.DropDownItems.Add(mnuItemDict);
                mnuItemDict.Enabled = bUserDictionaries;
            }

            // show the context menu at correct position
            contextMenuStrip1.Show(dataGridView1, 
                new Point(
                    e.X + dataGridView1.GetCellDisplayRectangle(
                        e.ColumnIndex, 
                        e.RowIndex, 
                        true).X,
                    e.Y + dataGridView1.GetCellDisplayRectangle(
                        e.ColumnIndex, 
                        e.RowIndex, 
                        true).Y));
        }

        // the user selected a suggestion
        // misspelled word will be replaced and marks the entry in the
        // context menu
        void item_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem mnuItemSuggestion = sender as ToolStripMenuItem;
            ToolStripMenuItem mnuItemOwner = 
                mnuItemSuggestion.OwnerItem as ToolStripMenuItem;

            mnuItemOwner.Font = new Font(mnuItemOwner.Font.FontFamily, 
                mnuItemOwner.Font.Size, 
                FontStyle.Strikeout);

            mnuItemOwner.ForeColor = Color.Green;
            mnuItemOwner.Tag = null;
            
            DataGridViewCell cell = mnuItemSuggestion.Tag as DataGridViewCell;
            cell.Value = ((string)cell.Value).Replace(mnuItemOwner.Text,
                mnuItemSuggestion.Text);

            ((ToolStripMenuItem)mnuItemSuggestion.OwnerItem).DropDownItems.Clear();

            if (ContainsSuggestions((ContextMenuStrip)mnuItemOwner.Owner) == false)
            {
                bCloseMenu = false;
                contextMenuStrip1.Close();
            }
        }

        private bool ContainsSuggestions(ContextMenuStrip item)
        {
            foreach (ToolStripItem mnu in item.Items)
            {
                if (mnu.Tag != null)
                    return true;
            }

            return false;
        }

        private void contextMenuStrip1_Closing(object sender, ToolStripDropDownClosingEventArgs e)
        {
            e.Cancel = bCloseMenu;
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            bCloseMenu = false;
            contextMenuStrip1.Close();
        }

        private void toolStripButton2_Click(object sender, EventArgs e)
        {
            frmSpelling dlgSpell = new frmSpelling(dataGridView1, txSpellChecker1);
            dlgSpell.ShowDialog();
        }
    }
}
